import { BrowserWindow } from 'electron';

/**
 * Ipc invoke decorator
 */
declare function IpcInvoke(event: string): MethodDecorator;
/**
 * Ipc on decorator
 */
declare function IpcOn(event: string, name?: string): MethodDecorator;
/**
 * Controller decorator, help to initialize controller
 */
declare function Controller(): ClassDecorator;
/**
 * Injectable decorator, help to inject service
 */
declare function Injectable(): ClassDecorator;
/**
 * Inject decorator, help to inject custom injectable item
 */
declare function Inject(name: string): ParameterDecorator;
/**
 * Window decorator, help to inject window
 */
declare function Window(name?: string): ParameterDecorator;

declare type Construct<T = any> = new (...args: Array<any>) => T;
declare type ControllerClass = Construct;
declare type InjectableClass = Construct;
/**
 * Window options
 */
interface WindowOpts {
    /**
     * Window's name, you can use @Window(name) to inject window
     */
    name: string;
    win: BrowserWindow;
}
/**
 * Custom injectable item options
 */
interface InjectableOpts {
    name: string;
    inject: any;
}
interface Options {
    /**
     * Windows will be created when app is ready
     */
    window: WindowOpts[] | (() => WindowOpts | Promise<WindowOpts>)[] | BrowserWindow | (() => BrowserWindow | Promise<BrowserWindow>);
    /**
     * Automatically initialized controller
     */
    controllers: ControllerClass[];
    /**
     * Custom injectable items
     */
    injects?: InjectableOpts[];
}
/**
 * Response of Ipc communication
 */
interface IpcResponse<T> {
    data: T;
    error?: any;
}
/**
 * Create and initialize Einf app
 */
declare function createEinf({ window, controllers, injects }: Options): Promise<void>;

export { Controller, ControllerClass, Inject, Injectable, InjectableClass, InjectableOpts, IpcInvoke, IpcOn, IpcResponse, Options, Window, WindowOpts, createEinf };
